package;

import entities.Bullet;
import entities.Enemy;
import entities.EnemySpawn;
import entities.EnemyZone;
import entities.HolyRoom;
import entities.PoisonCurtain;
import entities.PowerUp;
import entities.RandomHuman;
import entities.WaterPowerUp;
import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.group.FlxGroup.FlxTypedGroup;
import flixel.math.FlxPoint;
import flixel.system.FlxSound;
import flixel.text.FlxText;
import flixel.tile.FlxTile;
import flixel.tile.FlxTilemap;
import flixel.ui.FlxBar;
import flixel.ui.FlxButton;
import flixel.util.FlxColor;
import flixel.util.FlxSave;
import flixel.util.FlxStringUtil;
import flixel.util.FlxTimer;
import generator.Generator;
import helper.HealthBar;
import helper.PowerUpManager;
import helper.RoundManager;
import helper.Whistle;
import player.Player;
import ui.UIEnemyCount;
import ui.UIHolyRoom;
import ui.UIHolyWater;
import ui.UIHumanCount;
import ui.HowToPlayPage1;

class PlayState extends FlxState
{

	static inline var TILE_SIZE:Int = 16;

	public var map:FlxTilemap;
	var generator:Generator;
	var puManager:PowerUpManager;
	public var holyRoom:HolyRoom;
	public var player:Player;
	public var humans:FlxTypedGroup<RandomHuman>;
	public var spawns:FlxTypedGroup<EnemySpawn>;
	public var enemys:FlxTypedGroup<Enemy>;
	public var whistle:Whistle;
	public var bullets:FlxTypedGroup<Bullet>;
	public var powerUPs:FlxTypedGroup<PowerUp>;
	
	public var enemyZones:FlxTypedGroup<EnemyZone>;

	public var livingenemyCount:Int = 0;

	public var roundManager:RoundManager;

	var curtain:PoisonCurtain;
	var round:Int=0;

	var startTimer:FlxTimer;
	var startRoundTimer:FlxTimer;

	public var nextRoundTime:Int = 10;

	var gameOverSound:FlxSound;
	var gameIsEnding:Bool = false;

	public var started:Bool = false;

	public var enemyCount:Int=0;
	public var enemyTypes:Int = 0;
	public var enemyMultiply:Int = 0;

	var uiHW:UIHolyWater;
	var uiHC:UIHumanCount;
	var uiHR:UIHolyRoom;
	var uiEC:UIEnemyCount;
	var uiButtonStart:FlxButton;
	var uiButtonGenerate:FlxButton;
	var uiText:FlxText;

	public var uiPowerUpText:FlxText;
	public var uiPowerUpTimer:FlxTimer;

	var uiStartBar:FlxBar;
	var uiStartBarText:FlxText;

	public var evilTime:Bool = false;

	override public function create():Void
	{
		super.create();
		
		var save:FlxSave = new FlxSave();
		save.bind("howTo");
		if (!save.data.howTo)
		{
			openSubState(new HowTo());
		}

		FlxG.sound.volume = 0.5;

		puManager = new PowerUpManager(this);
		roundManager = new RoundManager(this);

		var cursor:FlxSprite = new FlxSprite();
		cursor.loadGraphic(AssetPaths.Cursor__png, false, 16, 16);

		loadTimer();

		FlxG.mouse.load(cursor.pixels);

		if (FlxG.sound.music == null || !FlxG.sound.music.playing) playMusic(false);

		bgColor = FlxColor.GRAY;

		FlxG.debugger.toggleKeys = ["P"];

		holyRoom = new HolyRoom(0, 0);
		add(holyRoom);

		spawns = new FlxTypedGroup<EnemySpawn>();
		add(spawns);

		map = new FlxTilemap();

		generator = new Generator();
		generator.generateMap();

		var csvData:String = FlxStringUtil.bitmapToCSV(generator.mapData);
		map.loadMapFromCSV(csvData, AssetPaths.tiles__png, TILE_SIZE, TILE_SIZE, AUTO);
		add(map);

		// Randomly pick room for player to start in
		var emptyTiles:Array<FlxPoint> = map.getTileCoords(0, false);
		var randomEmptyTile:FlxPoint = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		spawns.add(new EnemySpawn(randomEmptyTile.x, randomEmptyTile.y));

		whistle = new Whistle(0, 0);
		add(whistle);

		randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		player = new Player(randomEmptyTile.x, randomEmptyTile.y, whistle);
		add(player);

		player.setHealthBar(new HealthBar(player, FlxColor.GREEN));
		add(player.healthBar);

		humans = new FlxTypedGroup<RandomHuman>();
		spawnHumans();
		add(humans);

		enemys = new FlxTypedGroup<Enemy>();
		add(enemys);
		
		enemyZones = new FlxTypedGroup<EnemyZone>();
		add(enemyZones);

		bullets = new FlxTypedGroup<Bullet>();
		add(bullets);

		powerUPs = new FlxTypedGroup<PowerUp>();
		add(powerUPs);

		randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		curtain = new PoisonCurtain();
		add(curtain);
		add(curtain.emitter);
		addUI();

		loadSounds();
	}

	public function specialEvent(id:Int)
	{
		switch (id)
		{
			case 0:
			// nothing to do here
			case 1:
				spawnNewHuman();
			case 2:
				spawnNewEnemySpawn();
			case 3:
				spawnNewHuman();
				spawnNewEnemySpawn();
			default:
				// nothing to do here
		}
	}

	function getRandomPosition():FlxPoint
	{
		// Randomly pick room for player to start in
		var emptyTiles:Array<FlxPoint> = map.getTileCoords(0, false);
		var randomEmptyTile:FlxPoint = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		if ( randomEmptyTile == null)
		{
			randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];
		}

		return randomEmptyTile;
	}

	public function loadSounds()
	{
		#if flash
		gameOverSound = FlxG.sound.load(AssetPaths.gameOverLaughing__wav);
		#end

		#if (html5 || desktop)
		gameOverSound = FlxG.sound.load(AssetPaths.gameOverLaughing__ogg);
		#end

	}

	public function addPowerUp(?id:Int = 0)
	{
		if (id == 6)
		{
			return; // kein PowerUP
		}
		
		var powerUp:PowerUp = null;		
		if (id == 0)
		{
			powerUp = puManager.getRandomPowerUp();
		}
		else
		{
			powerUp = puManager.getPowerUp(id);
		}

		var position:FlxPoint = getRandomPosition();
		powerUp.setPosition(position.x, position.y);
		powerUPs.add(powerUp);
	}

	public function playMusic(actionTime:Bool)
	{
		if (actionTime)
		{
			#if flash
			FlxG.sound.playMusic(AssetPaths.FatherHustlerActoinTime__wav, 1, true);
			#end

			#if (html5 || desktop)
			FlxG.sound.playMusic(AssetPaths.FatherHustlerActionTime__ogg, 1, true);
			#end
		}

		else
		{
			#if flash
			FlxG.sound.playMusic(AssetPaths.FatherHustlerChilltime__wav, .5, true);
			#end
			#if (html5 || desktop)
			FlxG.sound.playMusic(AssetPaths.FatherHustlerChilltime__ogg, .5, true);
			#end
		}

	}

	public function disableTexts()
	{
		uiButtonStart.active = false;
		uiButtonStart.visible = false;
		uiButtonGenerate.active = false;
		uiButtonGenerate.visible = false;
		uiText.visible = false;
		uiText.active = false;
	}

	function spawnHumans()
	{
		var emptyTiles:Array<FlxPoint> = map.getTileCoords(0, false);
		var randomEmptyTile:FlxPoint = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		for (i in 0...10)
		{
			randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

			if (randomEmptyTile == null) randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];
			var human:RandomHuman = new RandomHuman(randomEmptyTile.x, randomEmptyTile.y);
			humans.add(human);

			human.setHealthBar(new HealthBar(human));
			add(human.healthBar);

		}
	}

	function setRoundSettings()
	{
		round ++;
		roundManager.prepareNextRound(round);

		
		//enemyCount = 0;
		//enemyTypes = 1;
		//enemyMultiply = 0;
		//nextRoundTime = 20;
		//spawnNewHuman();
		//spawnNewEnemySpawn();
		
		livingenemyCount = spawns.length * enemyCount * enemyMultiply;
	}

	function spawnNewHuman()
	{
		var emptyTiles:Array<FlxPoint> = map.getTileCoords(0, false);
		var randomEmptyTile:FlxPoint = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		if (randomEmptyTile == null) randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];
		var human:RandomHuman = new RandomHuman(randomEmptyTile.x, randomEmptyTile.y);
		humans.add(human);
	}

	function spawnNewEnemySpawn()
	{
		spawns.add(new EnemySpawn(0, 0));
	}

	function loadTimer()
	{
		startTimer = new FlxTimer();
		startRoundTimer = new FlxTimer();

		uiPowerUpTimer = new FlxTimer();
	}

	function startRound(timer:FlxTimer)
	{
		startRoundTimer.start(nextRoundTime, nextRound);
		prepareNextRound(false);
		disableTexts();
		if (!player.holyRoomSetted) player.setHolyRoom();
	}

	function prepareNextRound(?changeMusic:Bool=true)
	{
		if (changeMusic) playMusic(false);

		//addPowerUp();

		uiStartBar.parent = startRoundTimer;
		uiStartBar.visible = true;
		uiStartBarText.text = "Next round starts in... ";
		uiStartBarText.visible = true;
	}

	function nextRound(timer:FlxTimer)
	{
		setRoundSettings();
		uiStartBar.visible = false;
		uiStartBarText.visible = false;
		startEvilTime();
	}

	function addUI()
	{
		uiHW = new UIHolyWater(10, FlxG.height - 34, player);
		add(uiHW.uiGroup);

		uiHC = new UIHumanCount(FlxG.width - 34, FlxG.height - 34, humans);
		add(uiHC.uiGroup);

		uiHR = new UIHolyRoom(FlxG.width - 34, 2, player);
		add(uiHR.uiGroup);

		uiButtonStart = new FlxButton(200, 100, "Start Game", startGame);
		add(uiButtonStart);

		uiButtonGenerate = new FlxButton(400, 100, "Generate new", generateMap );
		add(uiButtonGenerate);

		uiEC = new UIEnemyCount(2, 2);
		add(uiEC.uiGroup);

		uiStartBar = new FlxBar(34, 2, FlxBarFillDirection.LEFT_TO_RIGHT, FlxG.width - 68, 32, startTimer, "progress", 0, 1, true);
		uiStartBar.createColoredEmptyBar(FlxColor.BLACK);
		uiStartBar.createColoredFilledBar(FlxColor.BLUE);
		uiStartBar.alpha = .8;
		uiStartBar.visible = false;

		add(uiStartBar);
		uiStartBarText = new FlxText(uiStartBar.x, uiStartBar.y, uiStartBar.barWidth, "Game starts in...", 32);
		uiStartBarText.font = AssetPaths.Joust__ttf;
		uiStartBarText.alignment = FlxTextAlign.CENTER;
		uiStartBarText.visible = false;
		add(uiStartBarText);

		uiPowerUpText = new FlxText(0, 0, 100, "", 10);
		uiPowerUpText.visible = false;
		add(uiPowerUpText);
		
		uiText = new FlxText(0, 10, FlxG.width, "Choose your destiny!", 32);
		uiText.font = AssetPaths.Joust__ttf;
		uiText.alignment = FlxTextAlign.CENTER;
		add(uiText);

	}

	function startGame()
	{
		started = true;
		startTimer.start(20, startRound);
		uiStartBar.visible = true;
		uiStartBarText.visible = true;
		disableTexts();
	}

	function generateMap()
	{
		FlxG.switchState(new PlayState());
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);

		testActions();

		FlxG.collide(map, player);
		FlxG.overlap(player, enemyZones, enemyAttack);
		FlxG.collide(humans, map);
		FlxG.collide(enemys);
		FlxG.collide(enemys, map);
		FlxG.collide(bullets, map, bulletHitsMap);
		FlxG.overlap(bullets, enemys, bulletHitsEnemy);
		FlxG.collide(player, enemys);
		FlxG.overlap(whistle, humans, comeOnHuman);
		FlxG.overlap(holyRoom, enemys, hurtRoom);
		FlxG.overlap(player, powerUPs, pickUpPowerUp);

		updateUI(elapsed);

		if (checkEvilTimeIsFinished() && enemys.countLiving() == 0)
		{
			stopEvilTime();
		}

		if (checkGameOver() && !gameIsEnding)
		{
			endGame();
		}
	}
	
	function enemyAttack(player:Player, enemyZone:EnemyZone)
	{
		if (enemyZone.active && enemyZone.alive)
		{
			enemyZone.attack();
			
		}
	}

	function pickUpPowerUp(player:Player, powerUp:PowerUp)
	{
		if (powerUp.alive && powerUp.active)
		{
			powerUp.doEffect(player);
		}
	}

	function checkGameOver()
	{
		if (humans.countLiving() > 0) return false;
		else return true;
	}

	function endGame()
	{
		FlxG.camera.fade(FlxColor.RED, 3, false, gameOverScreen, false);

		#if html5
		var sprite:FlxSprite = new FlxSprite(0, 0);
		sprite.makeGraphic(FlxG.width, FlxG.height, FlxColor.RED);
		add(sprite);
		#end

		gameIsEnding = true;
		gameOverSound.play();
	}

	function gameOverScreen()
	{
		FlxG.switchState(new EndState(round));
	}

	function hurtRoom(holyRoom:HolyRoom, enemy:Enemy)
	{
		if (holyRoom.alive && enemy.alive)
		{
			enemy.doDamage();
		}
	}

	function checkEvilTimeIsFinished():Bool
	{
		for (spawn in spawns)
		{
			if (!spawn.finished)
			{
				return false;
			}
		}

		return true;
	}

	function updateUI(elapsed:Float)
	{
		uiHW.update(elapsed);
		uiHC.update(elapsed);
		uiHR.update(elapsed);
		uiEC.update(elapsed);
	}

	function bulletHitsMap(bullet:Bullet, tile:FlxObject)
	{
		bullet.kill();
	}

	function bulletHitsEnemy(bullet:Bullet, enemy:Enemy)
	{
		if (bullet.animation.name != "explode")
		{
			bullet.kill();
			enemy.hurt(bullet.damage);
		}
	}

	function comeOnHuman(whistle:Whistle, human:RandomHuman)
	{
		if (whistle.whistled)
		{
			human.setSaved(false);
		}
	}

	function startEvilTime()
	{
		evilTime = true;

		playMusic(true);

		for (spawn in spawns)
		{
			spawn.startRound(enemyCount,enemyMultiply,enemyTypes);
		}
		curtain.startEvilTime();
	}

	public function stopEvilTime()
	{
		if ( evilTime)
		{
			evilTime = false;
			curtain.stopEvilTime();
			startRoundTimer.start(nextRoundTime, nextRound);
			prepareNextRound();
		}
	}

	function testActions()
	{
		if (FlxG.keys.anyPressed(["G"]))
		{
			if (!evilTime) startEvilTime();
			//else stopEvilTime();
		}
	}

}
