package entities;
import flixel.FlxG;
import flixel.math.FlxAngle;
import flixel.math.FlxPoint;
import flixel.system.FlxSound;
import flixel.util.FlxPath;
import flixel.util.FlxTimer;

/**
 * ...
 * @author Nicolas Korbel
 */
class Demon extends Enemy
{

	var SPEED:Float = 150;

	var enemyZone:EnemyZone;

	var attackedTimer:FlxTimer;
	var attackReady:Bool = true;

	var sound:FlxSound;

	public function new(?X:Float=0, ?Y:Float=0)
	{
		super(X, Y);
		loadGraphic(AssetPaths.Demon__png, true, 16, 16);

		maxHealth = 3;

		health = maxHealth;

		animation.add("idle", [0], 1, false);
		animation.add("walking", [0, 1], 6, true);
		animation.play("idle");

		attackedTimer = new FlxTimer();

		enemyZone = new EnemyZone(48, this);
		state.enemyZones.add(enemyZone);

		loadSounds();
	}

	override public function attack()
	{
		if (attackReady)
		{
			super.attack();
			attackReady = false;
			attackedTimer.start(2, newAttackReady);
		}
	}

	function newAttackReady(timer:FlxTimer)
	{
		attackReady = true;
	}

	function loadSounds()
	{
		#if flash
		sound = FlxG.sound.load(AssetPaths.demon__wav);
		#end

		#if (html5 || desktop)
		sound = FlxG.sound.load(AssetPaths.demon__ogg);
		#end
	}

	override public function kill():Void
	{
		super.kill();
		enemyZone.kill();

	}

	function findPathToPlayer()
	{
		if (!attackReady)
		{
			if (path != null) path.speed = 0;
		}
		else
		{
			var playerMidPoint:FlxPoint = state.player.getMidpoint();
			var midPoint:FlxPoint = getMidpoint();		

			path = new FlxPath().start(state.map.findPath(midPoint,playerMidPoint));
		}

	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);

		if (velocity.x != 0 || velocity.y != 0)
		{
			animation.play("walking");
		}
		else{
			animation.play("idle");
		}

		if (!isOnScreen(FlxG.camera)) kill();

		findPathToPlayer();

	}

	override public function revive():Void
	{
		super.revive();
		enemyZone.revive();
	}

	override public function startAction()
	{
		sound.play(true);
	}

}