package entities;

import flixel.FlxG;
import flixel.FlxSprite;
import flixel.math.FlxPoint;
import flixel.system.FlxAssets.FlxGraphicAsset;
import flixel.util.FlxTimer;

/**
 * ...
 * @author Nicolas Korbel
 */
class EnemySpawn extends FlxSprite
{

	static var spawnTime:Float = 2;

	public var finished:Bool = true;
	var count:Int = 0;
	var wavePosition:Int = 0;
	var multiply:Int = 1;
	var spawnTimer:FlxTimer;
	var enemyTypes:Int = 1;
	var postionFound:Bool = false;

	var state:PlayState;

	public function new(?X:Float=0, ?Y:Float=0)
	{
		super(X, Y);

		loadGraphic(AssetPaths.EnemySpawn__png, true, 32, 32);
		animation.add("idle", [0, 1, 2], 6, true);
		animation.play("idle");

		spawnTimer = new FlxTimer();

		state = cast FlxG.state;

		replaceSpawn();

	}

	public function startRound(count:Int, multiply:Int, enemyTypes:Int)
	{
		if (finished)
		{
			finished = false;
			wavePosition = 0;
			this.count = count;
			this.enemyTypes = enemyTypes;
			this.multiply = multiply;

			spawnEnemy();
		}
	}

	private function spawnEnemy()
	{
		for (i in 0...multiply)
		{
			newEnemy(state.roundManager.enemyArray[wavePosition]);
		}

		wavePosition++;

		spawnTimer.start(spawnTime, spawnNext);

	}

	function spawnNext(timer:FlxTimer)
	{
		if (count > wavePosition)
		{
			spawnEnemy();
		}
		else
		{
			finished = true;
		}
	}

	function replaceSpawn()
	{
		var emptyTiles:Array<FlxPoint> = state.map.getTileCoords(0, false);
		var randomEmptyTile:FlxPoint = emptyTiles[FlxG.random.int(0, emptyTiles.length)];
		if (randomEmptyTile == null) randomEmptyTile = emptyTiles[FlxG.random.int(0, emptyTiles.length)];

		x = randomEmptyTile.x;
		y = randomEmptyTile.y;

		if (FlxG.collide(this, state.map))
		{
			replaceSpawn();
		}

	}

	function newRandomEnemy():Enemy
	{

		var randomEnemy:Int = FlxG.random.int(1, enemyTypes);
		var enemy:Enemy;
		enemy = newEnemy(randomEnemy);
		return enemy;

	}

	function newEnemy(id:Int):Enemy
	{

		var enemy:Enemy;
		switch (id)
		{
			case 1:
				enemy = state.enemys.recycle(Zombie, newZombie, true, true);
			case 2:
				enemy = state.enemys.recycle(SuperZombie, newSuperZombie, true, true);
			case 3:
				enemy = state.enemys.recycle(Teleporter, newTeleporter, true, true);
			case 4:
				enemy = state.enemys.recycle(Demon, newDemon, true, true);
			default:
				enemy = state.enemys.recycle(Zombie, newZombie, true, true);

		}
		setEnemyOnRandomPosition(enemy);
		enemy.startAction();
		return enemy;

	}

	function setEnemyOnRandomPosition(enemy:Enemy)
	{
		enemy.setPosition(FlxG.random.float(x, x + 16), FlxG.random.float(y, y + 16));
	}

	function newZombie():Enemy
	{
		var zombie = new Zombie();
		state.enemys.add(zombie);
		return zombie;
	}

	function newDemon():Enemy
	{
		var demon = new Demon();
		state.enemys.add(demon);
		return demon;
	}

	function newTeleporter():Enemy
	{
		var teleporter = new Teleporter();
		state.enemys.add(teleporter);
		return teleporter;
	}

	function newSuperZombie():Enemy
	{
		var zombie = new SuperZombie();
		state.enemys.add(zombie);
		return zombie;
	}

}