package entities;

import flixel.FlxG;
import flixel.FlxObject;
import flixel.FlxSprite;
import flixel.math.FlxPoint;
import flixel.system.FlxAssets.FlxGraphicAsset;
import flixel.util.FlxPath;
import flixel.util.FlxTimer;
import helper.Entity;

/**
 * ...
 * @author Nicolas Korbel
 */
class RandomHuman extends Entity
{

	public static var WALKSPEED:Float = 20;
	public static var SPEED:Float = 120;
	
	public static var POISON_TIME:Float = 5;

	private var savedPosition:FlxPoint;
	private var saved:Bool;
	private var walkAgain:Bool;
	private var walkTimer:FlxTimer;
	private var lastVelocity:FlxPoint;
	private var poisonTimer:FlxTimer;	

	public function new(?X:Float=0, ?Y:Float=0)
	{
		super(X, Y);

		savedPosition = this.getPosition();
		saved = true;
		walkAgain = true;
		
		allowCollisions = FlxObject.ANY;

		walkTimer = new FlxTimer();
		walkTimer.finished = true;
		
		poisonTimer = new FlxTimer();

		loadGraphic("assets/images/Random" + FlxG.random.int(1, 4) + ".png", true, 16, 16);

		health = 5;

		animation.add("idle", [0],1, false);
		animation.add("walking", [0, 1], 6, true);

	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);
		
		checkPoison();

		if (velocity.x != 0 || velocity.y != 0)
		{
			animation.play("walking");
		}
		else{
			animation.play("idle");
		}

		if (saved && walkTimer.finished && walkAgain)
		{
			walkTimer.start(1, walkingFinished);
			walkAgain = false;
			lastVelocity = FlxPoint.get(FlxG.random.float(-1, 1) * WALKSPEED, FlxG.random.float(-1, 1) * WALKSPEED);
			velocity = lastVelocity;
		}

		if (!saved)
		{
			var state:PlayState = cast FlxG.state;
			path = new FlxPath().start([state.player.getPosition()], SPEED);
		}

	}
	
	function checkPoison()
	{
		if ( state.evilTime && !FlxG.overlap(this, state.holyRoom) && poisonTimer.finished)
		{
			hurt(1);
			poisonTimer.start(POISON_TIME);
		}
		else if( state.evilTime && !FlxG.overlap(this, state.holyRoom) && !poisonTimer.active)
		{
			poisonTimer.start(POISON_TIME);
		}
	}	

	function walkingFinished(timer:FlxTimer)
	{
		timer.start(1, walkBack);
		velocity.set(0, 0);
	}

	function walkBack(timer:FlxTimer)
	{
		timer.start(1, waitWalking);
		//lastVelocity.y = lastVelocity.y * -1;
		//lastVelocity.x =  lastVelocity.x * -1;

		path = new FlxPath([getPosition(), savedPosition]).start();
		path.speed = WALKSPEED;
	}

	function waitWalking(timer:FlxTimer)
	{
		timer.start(1);
		walkAgain = true;
		velocity.set(0, 0);
	}

	public function setSavedPostion()
	{
		savedPosition = getPosition();
	}

	public function setSaved(value:Bool)
	{
		saved = value;
		setSavedPostion();
	}

}