package entities;
import flixel.FlxG;
import flixel.math.FlxPoint;
import flixel.system.FlxSound;
import flixel.util.FlxPath;
import flixel.util.FlxTimer;

/**
 * ...
 * @author Nicolas Korbel
 */
class Teleporter extends Enemy
{

	static var TELEPORT_TIME:Float = 1.5;

	var teleportPath:Array<FlxPoint>;
	var teleportTimer:FlxTimer;
	var enemyZone:EnemyZone;
	var sound:FlxSound;

	public function new(?X:Float=0, ?Y:Float=0)
	{
		super(X, Y);
		loadGraphic(AssetPaths.Teleporter__png, true, 16, 16);

		maxHealth = 3;
		set_immovable(true);

		teleportTimer = new FlxTimer();

		health = maxHealth;

		animation.add("idle", [0, 1], 6, true);
		animation.play("idle");

		enemyZone = new EnemyZone(40, this);
		state.enemyZones.add(enemyZone);

		loadSounds();
	}

	function loadSounds()
	{
		#if flash
		sound = FlxG.sound.load(AssetPaths.Teleport__wav);
		#end

		#if (html5 || desktop)
		sound = FlxG.sound.load(AssetPaths.Teleport__ogg);
		#end
	}

	override public function kill():Void
	{
		super.kill();
		teleportTimer.finished = true;
	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);
		velocity.set(0, 0);

	}

	override public function revive():Void
	{
		super.revive();
		
		enemyZone.revive();
	}

	override public function startAction()
	{
		teleportPath = state.map.findPath(getMidpoint(), state.holyRoom.getMidpoint());
		teleportTimer.start(TELEPORT_TIME, teleport);
	}

	function teleport(timer:FlxTimer)
	{
		if (teleportPath.length != 0 && alive)
		{
			var point:FlxPoint = teleportPath.shift();
			sound.play(true);
			setPosition(point.x, point.y);
			timer.start(TELEPORT_TIME, teleport);
		}
	}

}