package entities;

import flixel.FlxG;
import flixel.system.FlxSound;
import flixel.util.FlxPath;
import flixel.util.FlxTimer;
import helper.Entity;

/**
 * ...
 * @author Nicolas Korbel
 */
class Zombie extends Enemy
{
	static var SPEED:Float = 40;

	static var SOUNDTIME:Float = 3;
	var sound1:FlxSound;
	var sound2:FlxSound;
	var sound3:FlxSound;
	var sound4:FlxSound;
	var soundTimer:FlxTimer;
	
	var enemyZone:EnemyZone;

	public function new(?X:Float=0, ?Y:Float=0)
	{
		super(X, Y);

		loadGraphic("assets/images/Zombie" + FlxG.random.int(1, 4) + ".png", true, 16, 16);

		maxHealth = 2;

		health = maxHealth;

		animation.add("idle", [0],1, false);
		animation.add("walking", [0, 1], 6, true);

		soundTimer = new FlxTimer();
		soundTimer.start(SOUNDTIME, makeSomeNoise);

		enemyZone = new EnemyZone(40, this);
		state.enemyZones.add(enemyZone);

		loadSounds();
	}

	function makeSomeNoise(timer:FlxTimer)
	{
		if (!alive) return;
		timer.start(SOUNDTIME, makeSomeNoise);

		var random:Int = FlxG.random.int(0, 3);
		switch (random)
		{
			case 0:
				sound1.play(true);
			case 1:
				sound2.play(true);
			case 2:
				sound3.play(true);
			case 3:
				sound4.play(true);
			default:
				sound1.play(true);
		}
	}

	function loadSounds()
	{
		#if flash
		sound1 = FlxG.sound.load(AssetPaths.zombie1__wav,.5);
		sound2 = FlxG.sound.load(AssetPaths.zombie2__wav,.5);
		sound3 = FlxG.sound.load(AssetPaths.zombie3__wav,.5);
		sound4 = FlxG.sound.load(AssetPaths.zombie4__wav,.5);
		#end

		#if (html5 || desktop)
		sound1 = FlxG.sound.load(AssetPaths.zombie1__ogg,.5);
		sound2 = FlxG.sound.load(AssetPaths.zombie2__ogg,.5);
		sound3 = FlxG.sound.load(AssetPaths.zombie3__ogg,.5);
		sound4 = FlxG.sound.load(AssetPaths.zombie4__ogg,.5);
		#end
	}

	override public function kill():Void
	{
		super.kill();

		soundTimer.active = false;

		sound1.stop();
		sound2.stop();
		sound3.stop();
		sound4.stop();

	}

	override public function update(elapsed:Float):Void
	{
		super.update(elapsed);

		if (velocity.x != 0 || velocity.y != 0)
		{
			animation.play("walking");
		}
		else{
			animation.play("idle");
		}
	}

	override public function revive():Void
	{
		super.revive();
		
		soundTimer.active = true;
		soundTimer.start(6, makeSomeNoise);
		enemyZone.revive();
	}

	override public function startAction()
	{
		path = new FlxPath().start(state.map.findPath(getMidpoint(), state.holyRoom.getMidpoint()), SPEED);
	}

}