package player;

import entities.HolyWater;
import entities.RandomHuman;
import flash.utils.Timer;
import flixel.FlxG;
import flixel.FlxSprite;
import flixel.FlxState;
import flixel.math.FlxAngle;
import flixel.math.FlxPoint;
import flixel.math.FlxVelocity;
import flixel.system.FlxAssets.FlxGraphicAsset;
import flixel.input.keyboard.FlxKey;
import flixel.system.FlxSound;
import flixel.util.FlxColor;
import flixel.util.FlxTimer;
import helper.Entity;
import helper.HealthBar;
import helper.Whistle;

/**
 * ...
 * @author Nicolas Korbel
 */
class Player extends Entity
{
	static var upKeys:Array<FlxKey> = ["W", "UP"];
	static var downKeys:Array<FlxKey> = ["S", "DOWN"];
	static var leftKeys:Array<FlxKey> = ["A", "LEFT"];
	static var rightKeys:Array<FlxKey> = ["D", "RIGHT"];

	static var SPEED:Float = 120;

	public var holyWaterCount:Int = 4;
	public var holyWaterMaxCount:Int = 4;
	public var refreshTime:Float = .75;
	public var holyWaterDamage:Int = 1;
	var refreshWaterTimer:FlxTimer;

	var respawnTimer:FlxTimer;

	var whistleSound:FlxSound;
	var halelujaSound:FlxSound;
	var throwSound:FlxSound;
	var stayHereSound:FlxSound;
	var dieSound:FlxSound;
	var respawnSound:FlxSound;

	public var holyRoomCount:Int = 1;
	public var holyRoomSetted:Bool = false;

	public var whistle:Whistle;

	var whistleTimer:FlxTimer;

	public function new(?X:Float=0, ?Y:Float=0, whistle:Whistle)
	{
		super(X, Y);
		loadGraphic(AssetPaths.Player__png, true, 16, 16);

		health = 5;

		loadSounds();

		refreshWaterTimer = new FlxTimer();
		refreshWaterTimer.start(2, refreshWater);

		respawnTimer = new FlxTimer();

		this.whistle = whistle;
		whistleTimer = new FlxTimer();
		whistleTimer.finished = true;

		animation.add("idle", [0],1, false);
		animation.add("walking", [0, 1], 6, true);

	}

	function refreshWater(timer: FlxTimer)
	{
		timer.start(refreshTime, refreshWater);
		if (holyWaterCount < holyWaterMaxCount)
		{
			holyWaterCount += 1;
		}
	}

	function loadSounds()
	{
		#if flash
		throwSound = FlxG.sound.load(AssetPaths.throw__wav,.5);
		whistleSound = FlxG.sound.load(AssetPaths.whistle__wav);
		halelujaSound = FlxG.sound.load(AssetPaths.haleluja__wav,.5);
		stayHereSound = FlxG.sound.load(AssetPaths.stayHere__wav);
		dieSound = FlxG.sound.load(AssetPaths.Dying__wav);
		respawnSound = FlxG.sound.load(AssetPaths.respawn__wav);
		#end

		#if (html5 || desktop)
		throwSound = FlxG.sound.load(AssetPaths.throw__ogg,.5);
		whistleSound = FlxG.sound.load(AssetPaths.whistle__ogg);
		halelujaSound = FlxG.sound.load(AssetPaths.haleluja__ogg,.5);
		stayHereSound = FlxG.sound.load(AssetPaths.stayHere__ogg);
		dieSound = FlxG.sound.load(AssetPaths.Dying__ogg);
		respawnSound = FlxG.sound.load(AssetPaths.respawn__ogg);
		#end
	}

	override public function kill():Void
	{
		super.kill();
		dieSound.play();
		respawnTimer.start(10, respawn);

	}
	
	function respawn(timer:FlxTimer)
	{
		health = 5;
		var holyRoomMid:FlxPoint = state.holyRoom.getMidpoint();
		x = holyRoomMid.x;
		y = holyRoomMid.y;
		revive();
		respawnSound.play();
	}
	
	override public function hurt(Damage:Float):Void 
	{
		super.hurt(Damage);
		FlxG.camera.shake(0.01, 0.2);
	}

	override public function update(elapsed:Float):Void
	{

		if (!state.started) return;
		movement();
		actions();
		mouseActions();
		whistle.x = x -56;
		whistle.y = y -56;
		super.update(elapsed);
	}

	function mouseActions()
	{
		if (FlxG.mouse.justPressed)
		{
			if (holyWaterCount > 0)
			{
				var holyWater:HolyWater = cast state.bullets.recycle(HolyWater, createHolyWater, true, true);
				var midPoint = getMidpoint();

				holyWater.setPosition(midPoint.x, midPoint.y);

				var shootAngle:Float = FlxAngle.angleBetweenMouse(this, true);
				holyWater.velocity = FlxVelocity.velocityFromAngle(shootAngle, SPEED);

				holyWater.shoot(holyWaterDamage);

				holyWaterCount -= 1;

				throwSound.play();
			}
		}

	}

	function createHolyWater():HolyWater
	{
		var holyWater:HolyWater = new HolyWater();
		state.bullets.add(holyWater);
		return holyWater;
	}

	function actions()
	{
		if (FlxG.keys.pressed.E)
		{
			pickUpWhistle();

		}

		if (FlxG.keys.pressed.Q)
		{
			leaveHuman();
		}

		if (FlxG.keys.pressed.SPACE)
		{
			setHolyRoom();

		}

		if (FlxG.keys.pressed.ENTER)
		{
			setHolyRoom();
		}

	}

	public function setHolyRoom()
	{
		if (holyRoomCount > 0)
		{
			state.holyRoom.setRoom(x - 56, y - 56);
			state.disableTexts();
			holyRoomSetted = true;
			halelujaSound.play();
			holyRoomCount -= 1;
		}
	}

	function pickUpWhistle()
	{
		if (whistleTimer.finished)
		{
			whistleTimer.start(2);
			whistleSound.play();
			whistle.blowMyWhistle();
		}
	}

	function leaveHuman()
	{
		var state:PlayState = cast FlxG.state;
		stayHereSound.play();
		for (human in state.humans)
		{
			human.setSaved(true);
		}
	}

	function movement()
	{

		if (FlxG.keys.anyPressed(upKeys))
		{
			velocity.y = -SPEED;
		}

		else if (FlxG.keys.anyPressed(downKeys))
		{
			velocity.y = SPEED;
		}

		else
		{
			velocity.y = 0;
		}

		if (FlxG.keys.anyPressed(leftKeys))
		{
			velocity.x = -SPEED;
		}

		else if (FlxG.keys.anyPressed(rightKeys))
		{
			velocity.x = SPEED;
		}

		else
		{
			velocity.x = 0;
		}

		if (velocity.x != 0 || velocity.y != 0)
		{
			animation.play("walking");
		}
		else
		{
			animation.play("idle");
		}
	}

}